package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.request.service.auth.UndergraduateLogin;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import cn.wustlinghang.wusthelper.internal.physics.exception.PhysicsRpcException;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class LoginService {
    private final RequestClientOption option;
    private final UndergraduateLogin undergraduateLogin;

    public LoginService(RequestClientOption option,
                        UndergraduateLogin undergraduateLogin) {
        this.option = option;
        this.undergraduateLogin = undergraduateLogin;
    }

    public String login(String username, String password, boolean legacy) throws PhysicsRpcException {
        try {
            if (legacy) {
                return undergraduateLogin.getLoginCookieLegacy(username, password, option);
            } else {
                return undergraduateLogin.getLoginCookie(username, password, option);
            }
        } catch (ApiException e) {
            throw wrapApiException(e);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.AUTH,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    public boolean verify(String cookie) throws PhysicsRpcException {
        try {
            return !undergraduateLogin.checkCookiesFail(cookie);
        } catch (IOException e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.AUTH,
                    PhysicsRpcException.ErrorCode.NETWORK_ERROR
            );
        }
    }

    private PhysicsRpcException wrapApiException(ApiException e) {
        PhysicsRpcException.ErrorCode errorCode = switch (e.getCode()) {
            case NETWORK_EXCEPTION -> PhysicsRpcException.ErrorCode.NETWORK_ERROR;
            case UNI_LOGIN_PASSWORD_WRONG -> PhysicsRpcException.ErrorCode.AUTH_PASSWORD_WRONG;
            case UNI_LOGIN_USER_NOT_EXISTS -> PhysicsRpcException.ErrorCode.AUTH_USER_NOT_EXISTS;
            case UNI_LOGIN_USER_BANNED -> PhysicsRpcException.ErrorCode.AUTH_USER_BANNED;
            case UNI_LOGIN_USER_DISABLED -> PhysicsRpcException.ErrorCode.AUTH_USER_DISABLED;
            case UNI_LOGIN_NEED_CHANGE_PASSWORD -> PhysicsRpcException.ErrorCode.AUTH_NEED_CHANGE_PASSWORD;
            case UNI_LOGIN_USER_NOT_ONLY -> PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            case UNI_LOGIN_NO_REGISTER -> PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            case UNI_LOGIN_NEED_TFA -> PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            default -> {
                log.error("本科生：登录代理请求异常，异常未处理");
                log.error("异常：", e);
                yield PhysicsRpcException.ErrorCode.AUTH_UNKNOWN_ERROR;
            }
        };

        return new PhysicsRpcException(
                PhysicsRpcException.TypeCode.NETWORK_EXCEPTION,
                PhysicsRpcException.SubModuleCode.AUTH,
                errorCode
        );
    }
}
