package cn.wustlinghang.wusthelper.internal.physics.services;

import cn.wustlinghang.mywust.core.parser.physics.PhysicsCoursePageParser;
import cn.wustlinghang.mywust.data.global.Course;
import cn.wustlinghang.mywust.data.global.Score;
import cn.wustlinghang.mywust.data.global.StudentInfo;
import cn.wustlinghang.mywust.data.physics.PhysicsCourse;
import cn.wustlinghang.mywust.data.undergrad.ExamDelayApplication;
import cn.wustlinghang.wusthelper.internal.physics.exception.PhysicsRpcException;
import cn.wustlinghang.wusthelper.rpc.exception.RpcException;
import jakarta.enterprise.context.ApplicationScoped;

import java.util.List;

@ApplicationScoped
public class ParseService {
    private final PhysicsCoursePageParser coursePageParser;

    public List<PhysicsCourse> parseCourseTable(String data) throws PhysicsRpcException {
        try {
            return coursePageParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.COURSE_TABLE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<Score> parseScore(String data) throws RpcException {
        try {
            if (data.contains("评教")) {
                throw new PhysicsRpcException(
                        PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                        PhysicsRpcException.SubModuleCode.SCORE,
                        PhysicsRpcException.ErrorCode.NEED_EVALUATE
                );
            }

            return scoreParser.parse(data);

        } catch (PhysicsRpcException e) {
            throw e;
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.SCORE,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public StudentInfo parseStudentInfo(String data) throws RpcException {
        try {
            return studentInfoPageParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.STUDENT_INFO,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public String parseTrainingPlan(String data) throws RpcException {
        try {
            return trainingPlanPageParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.TRAINING_PLAN,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public String parseCreditStatus(String data) throws RpcException {
        try {
            return creditStatusParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.CREDIT_STATUS,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }

    public List<ExamDelayApplication> parseExamDelayApplications(String data) throws RpcException {
        try {
            return examDelayParser.parse(data);
        } catch (Exception e) {
            throw new PhysicsRpcException(
                    PhysicsRpcException.TypeCode.PARSE_EXCEPTION,
                    PhysicsRpcException.SubModuleCode.EXAM_DELAY_APPLICATION,
                    PhysicsRpcException.ErrorCode.PARSE_ERROR
            );
        }
    }
}
